@@ -16,12 +16,17 @@ from Queue import Empty, Queue as _Queue
 
 from kombu import exceptions
 from kombu.transport import get_transport_cls
-from kombu.simple import SimpleQueue, SimpleBuffer
 from kombu.utils import retry_over_time
 from kombu.utils.compat import OrderedDict
 from kombu.utils.functional import wraps
 
 
+#: Connection info -> URI
+URI_FORMAT = """\
+%(transport)s://%(userid)s@%(hostname)s%(port)s%(virtual_host)s\
+"""
+
+
 class BrokerConnection(object):
     """A connection to the broker.
 
@@ -60,6 +65,8 @@ class BrokerConnection(object):
         >>> conn.release()
 
     """
+    URI_FORMAT = URI_FORMAT
+
     port = None
     virtual_host = "/"
     connect_timeout = 5
@@ -188,10 +195,9 @@ class BrokerConnection(object):
                 try:
                     return fun(*args, **kwargs)
                 except self.connection_errors + self.channel_errors, exc:
-                    if got_connection:
+                    if got_connection or \
+                            max_retries and retries > max_retries:
                         raise
-                    if max_retries and retries > max_retries:
-                        raise exceptions.EnsureExhausted()
                     errback and errback(exc, 0)
                     self._connection = None
                     self.close()
@@ -238,6 +244,16 @@ class BrokerConnection(object):
                             ("transport", transport_cls),
                             ("connect_timeout", self.connect_timeout)))
 
+    def as_uri(self):
+        fields = self.info()
+        port = fields["port"]
+        if port:
+            fields["port"] = ":%s" % (port, )
+        vhost = fields["virtual_host"]
+        if not vhost.startswith('/'):
+            fields["virtual_host"] = '/' + vhost
+        return self.URI_FORMAT % fields
+
     def Pool(self, limit=None, preload=None):
         """Pool of connections.
 
@@ -293,7 +309,7 @@ class BrokerConnection(object):
         return ChannelPool(self, limit, preload)
 
     def SimpleQueue(self, name, no_ack=None, queue_opts=None,
-            exchange_opts=None, channel=None):
+            exchange_opts=None, channel=None, **kwargs):
         """Create new :class:`~kombu.simple.SimpleQueue`, using a channel
         from this connection.
 
@@ -315,15 +331,17 @@ class BrokerConnection(object):
            object.
 
         """
+        from kombu.simple import SimpleQueue
+
         channel_autoclose = False
         if channel is None:
             channel = self.channel()
             channel_autoclose = True
         return SimpleQueue(channel, name, no_ack, queue_opts, exchange_opts,
-                           channel_autoclose=channel_autoclose)
+                           channel_autoclose=channel_autoclose, **kwargs)
 
     def SimpleBuffer(self, name, no_ack=None, queue_opts=None,
-            exchange_opts=None, channel=None):
+            exchange_opts=None, channel=None, **kwargs):
         """Create new :class:`~kombu.simple.SimpleQueue` using a channel
         from this connection.
 
@@ -333,22 +351,21 @@ class BrokerConnection(object):
         and acknowledgements are disabled (``no_ack``).
 
         """
+        from kombu.simple import SimpleBuffer
+
         channel_autoclose = False
         if channel is None:
             channel = self.channel()
             channel_autoclose = True
         return SimpleBuffer(channel, name, no_ack, queue_opts, exchange_opts,
-                            channel_autoclose=channel_autoclose)
+                            channel_autoclose=channel_autoclose, **kwargs)
 
     def _establish_connection(self):
         return self.transport.establish_connection()
 
     def __repr__(self):
         """``x.__repr__() <==> repr(x)``"""
-        info = self.info()
-        return "<BrokerConnection: %s>" % (
-                    ", ".join("%s=%r" % (item, info[item])
-                                for item in info.keys()[:8]))
+        return "<BrokerConnection: %s>" % self.as_uri()
 
     def __copy__(self):
         """``x.__copy__() <==> copy(x)``"""
@@ -450,6 +467,9 @@ class Resource(object):
     def prepare(self, resource):
         return resource
 
+    def close_resource(self, resource):
+        resource.close()
+
     def release(self, resource):
         """Release resource so it can be used by another thread.
 
@@ -461,6 +481,33 @@ class Resource(object):
         self._dirty.discard(resource)
         self._resource.put_nowait(resource)
 
+    def force_close_all(self):
+        """Closes and removes all resources in the pool (also those in use).
+
+        Can be used to close resources from parent processes
+        after fork (e.g. sockets/connections).
+
+        """
+        dirty = self._dirty
+        resource = self._resource
+        while 1:
+            try:
+                dres = dirty.pop()
+            except KeyError:
+                break
+            self.close_resource(dres)
+
+        resource.mutex.acquire()
+        try:
+            while 1:
+                try:
+                    res = resource.queue.pop()
+                except IndexError:
+                    break
+                self.close_resource(res)
+        finally:
+            resource.mutex.release()
+
 
 class ConnectionPool(Resource):
     LimitExceeded = exceptions.ConnectionLimitExceeded
@@ -474,11 +521,12 @@ class ConnectionPool(Resource):
         return copy(self.connection)
 
     def setup(self):
-        for i in xrange(self.limit):
-            conn = self.new()
-            if i < self.preload:
-                conn.connect()
-            self._resource.put_nowait(conn)
+        if self.limit:
+            for i in xrange(self.limit):
+                conn = self.new()
+                if i < self.preload:
+                    conn.connect()
+                self._resource.put_nowait(conn)
 
     def prepare(self, resource):
         if not resource._connection:
