@@ -13,6 +13,7 @@ from itertools import count
 from kombu.compression import compress
 from kombu.entity import Exchange, Queue
 from kombu.serialization import encode
+from kombu.syn import blocking as _SYN
 from kombu.utils import maybe_list
 
 
@@ -68,9 +69,8 @@ class Producer(object):
         if auto_declare is not None:
             self.auto_declare = auto_declare
 
-        if self.exchange:
-            self.exchange = self.exchange(self.channel)
-            self.auto_declare and self.declare()
+        self.exchange = self.exchange(self.channel)
+        self.auto_declare and self.declare()
 
         if self.on_return:
             self.channel.events["basic_return"].append(self.on_return)
@@ -269,7 +269,12 @@ class Consumer(object):
 
     def cancel_by_queue(self, queue):
         """Cancel consumer by queue name."""
-        self.channel.basic_cancel(self._active_tags[queue])
+        try:
+            tag = self._active_tags.pop(queue)
+        except KeyError:
+            pass
+        else:
+            self.channel.basic_cancel(tag)
 
     def purge(self):
         """Purge messages from all queues.
@@ -320,9 +325,9 @@ class Consumer(object):
           Currently not supported by RabbitMQ.
 
         """
-        return self.channel.basic_qos(prefetch_size,
-                                      prefetch_count,
-                                      apply_global)
+        return _SYN(self.channel.basic_qos, prefetch_size,
+                                            prefetch_count,
+                                            apply_global)
 
     def recover(self, requeue=False):
         """Redeliver unacknowledged messages.
@@ -336,7 +341,7 @@ class Consumer(object):
           delivering it to an alternative subscriber.
 
         """
-        return self.channel.basic_recover(requeue=requeue)
+        return _SYN(self.channel.basic_recover, requeue=requeue)
 
     def receive(self, body, message):
         """Method called when a message is received.
